module RoboRace.UI where

import Control.Applicative
import Control.Monad
import Data.Char
import Data.Maybe
import System.IO

import RoboRace.Game

uiMain :: String -> Int -> IO ()
uiMain server port = do
  do
    (game,state) <- start (server ++ ":" ++ show port)
    turnLoop game state
  `catch` \err ->
    putStrLn $ "An error happened: " ++ show err

turnLoop :: Game -> GameState -> IO ()
turnLoop game state = do
  case state of
    Won moves -> do
      putStrLn $ "You won in " ++ show moves ++ " moves"
    Error msg -> do
      putStrLn $ "An error: " ++ msg
    turn -> do
      putStrLn "Map: "
      showMap (trField turn) (trPosition turn) (trRotation turn)
      putStr "Available moves: "
      putStrLn $ show $ trMoves turn
      putStr "Write 5 moves: "
      hFlush stdout
      input <- getLine
      if length input > 5
        then do
          putStrLn "Write 5 (or less) moves, please   (press Enter)"
          getLine
          turnLoop game state
        else do
          let mbMoves = map (charToMove . toUpper) input
          if all isJust mbMoves
            then do
              let moves = map (\(Just x) -> x) mbMoves
              newState <- RoboRace.Game.turn game state moves
              turnLoop game newState
              return ()
            else do
              putStrLn "Invalid move   (press Enter)"
              getLine
              turnLoop game state

showMap :: GameField -> (Int,Int) -> Rotation -> IO ()
showMap field pos rot = do
  forM_ (zip [1..] field) $ \(y,elems) -> do
    forM_ (zip [1..] elems) $ \(x,elem) -> do
      if pos == (x,y)
        then case rot of
          North -> putChar '^'
          West -> putChar '<'
          South -> putChar 'v'
          East -> putChar '>'
        else putStr $ show elem
    putStrLn ""

